<?php
/**
 * @package            Joomla
 * @subpackage         Event Booking
 * @author             Tuan Pham Ngoc
 * @copyright          Copyright (C) 2010 - 2019 Ossolution Team
 * @license            GNU/GPL, see LICENSE.php
 */

defined('_JEXEC') or die;

class os_quickpay extends RADPayment
{
	
	/**
	 * Process Payment
	 *
	 * @param EventbookingTableRegistrant $row
	 * @param array                       $data
	 */
	public function processPayment($row, $data)
	{
		require_once JPATH_ROOT . '/components/com_eventbooking/payments/quickpay/helper.php';

		$app     = JFactory::getApplication();
		$Itemid  = $app->input->getInt('Itemid');
		$siteUrl = JUri::base();

		$amount      = round($data['amount'], 2);
		$amount      = $amount * 100;
		$orderNumber = sprintf('%04u', $row->id);

		$prefix = $this->params->get('prefix');
		

		if ($prefix)
		{
			$orderNumber = $prefix . $orderNumber;
		}

		$this->setParameter('agreement_id', $this->params->get('agreement_id'));
		$this->setParameter('amount', $amount);

		if ($row->process_deposit_payment)
		{
			$orderNumber = 'rp_' . $orderNumber;

			if (JPluginHelper::isEnabled('system', 'cache'))
			{
				$this->setParameter('continueurl', htmlspecialchars_decode($siteUrl . 'index.php?option=com_eventbooking&view=payment&layout=complete&Itemid=' . $Itemid . '&pt=' . time()));
			}
			else
			{
				$this->setParameter('continueurl', htmlspecialchars_decode($siteUrl . 'index.php?option=com_eventbooking&view=payment&layout=complete&Itemid=' . $Itemid));
			}
		}
		else
		{
			$this->setParameter('continueurl', htmlspecialchars_decode($siteUrl . 'index.php?option=com_eventbooking&view=complete&Itemid=' . $Itemid));
		}

		$this->setParameter('cancelurl', htmlspecialchars_decode($siteUrl . 'index.php?option=com_eventbooking&view=cancel&layout=default&id=' . $row->id . '&Itemid=' . $Itemid));
		$this->setParameter('callbackurl', htmlspecialchars_decode($siteUrl . 'index.php?option=com_eventbooking&task=payment_confirm&payment_method=os_quickpay'));
		$this->setParameter('language', $this->params->get('language'));
		$this->setParameter('autocapture', $this->params->get('autocapture'));
		$this->setParameter('autofee', $this->params->get('autofee'));
		$this->setParameter('payment_methods', $this->params->get('payment_methods'));
		$this->setParameter('branding_id', $this->params->get('branding_id'));
		$this->setParameter('google_analytics_tracking_id', $this->params->get('google_analytics_tracking_id'));
		$this->setParameter('google_analytics_client_id', $this->params->get('google_analytics_client_id'));
		$this->setParameter('product_id', $this->params->get('product_id'));
		$this->setParameter('category', $this->params->get('category'));

		$helper = new QuickpayHelper();
		$helper->setApiKey($this->params->get('api_key'));
		$result     = $helper->qpCreatePayment($orderNumber, $data['currency']);
		$result     = $helper->qpCreatePaymentLink($result->id, $this->parameters);
		$paymentUrl = $result->url;
			?>
        <div class="contentheading"><?php echo JText::_('Please wait while redirecting to Quickpay for completing the payment'); ?></div>
        <script type="text/javascript">
            function redirect() {
                window.location.href = "<?php echo $paymentUrl; ?>";
            }

            setTimeout('redirect()', 1000);
        </script>
		<?php
	}

	/**
	 * Validate the post data from quickpay to our server
	 *
	 * @return string
	 */
	private function validate()
	{
		$requestBody = file_get_contents("php://input");

		$this->notificationData = [];
		$this->logGatewayData($requestBody);

		$key = $this->params->get('private_key');

		$checksum = hash_hmac("sha256", $requestBody, $key);

		if ($checksum == $_SERVER["HTTP_QUICKPAY_CHECKSUM_SHA256"])
		{
			$request       = json_decode($requestBody);
			$operation     = end($request->operations);
			$orderId       = $request->order_id;
			$transactionId = $request->id;
			$prefix        = $this->params->get('prefix');

			if (strpos($orderId, 'rp_') !== false)
			{
				$orderId = substr($orderId, strlen('rp_'));
			}
			
			if ($prefix)
			{
				$orderId = substr($orderId, strlen($prefix));
			}

			$orderId = (int) $orderId;
			$amount  = $operation->amount / 100.0;

			if ($request->accepted && $operation->type == 'authorize' && $operation->qp_status_code == "20000")
			{
				$this->notificationData['id']     = $orderId;
				$this->notificationData['amount'] = $amount;
				$this->notificationData['txn_id'] = $transactionId;

				return true;
			}
		}

		return false;
	}

	/**
	 * Process payment
	 *
	 */
	public function verifyPayment()
	{
		if ($this->validate())
		{
			$id            = $this->notificationData['id'];
			$transactionId = $this->notificationData['txn_id'];
			$amount        = $this->notificationData['amount'];

			if ($amount < 0)
			{
				return false;
			}

			$row = JTable::getInstance('Registrant', 'EventbookingTable');

			if (!$row->load($id))
			{
				return false;
			}

			if ($row->published == 1 && $row->payment_status)
			{
				return false;
			}

			$this->onPaymentSuccess($row, $transactionId);
		}
	}
}
